#!/usr/bin/env python
# encoding: utf-8
#
# FreiTAG - A simple command line tool to tag and rename mp3s.
# Copyright (c) 2010-2011 Giuseppe Capizzi
# mailto: g.capizzi@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from re import sub
from string import capwords

from freitag import FreiTemplate, DEFAULT_FORMAT


class Operation:

    def apply(self, song):
        raise NotImplementError


class SetOperation(Operation):

    def __init__(self, tags):
        self.tags = tags

    def apply(self, song):
        song.update(self.tags)


class RenameOperation(Operation):

    def __init__(self, template=FreiTemplate(DEFAULT_FORMAT)):
        self.template = template

    def _format(self, song):
        return self.template.safe_substitute(song).strip()

    def apply(self, song):
        """Rename song according to the specified format."""
        song.filename = self._format(song)


class ExtractOperation(Operation):

    def __init__(self, template=FreiTemplate(DEFAULT_FORMAT)):
        self.template = template

    def apply(self, song):
        tags = self.template.extract(song.filename)
        song.update(tags)


class HumanizeOperation(Operation):

    def apply(self, song):
        """Humanize album, title and artist tags from tags dictionary."""
        tags_to_humanize = ['album', 'artist', 'title']

        for tag in tags_to_humanize:
            if tag in song:
                song[tag] = self._humanize_tag(song[tag])

    def _humanize_tag(self, string):
        return capwords(string.replace('_', ' '))



#!/usr/bin/env python
# encoding: utf-8
#
# FreiTAG - A simple command line tool to tag and rename mp3s.
# Copyright (c) 2010-2011 Giuseppe Capizzi
# mailto: g.capizzi@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from mutagen.mp3 import EasyMP3

from freitag import FreiSong, FreiTemplate
from freitag.operations import (SetOperation, RenameOperation,
                                ExtractOperation, HumanizeOperation)


class Command:

    def get_songs(self, files):
        return [FreiSong(EasyMP3(file)) for file in files]

    def run(self, args):
        raise NotImplementError


class GetCommand(Command):

    def run(self, args):
        songs = self.get_songs(args.files)
        template = FreiTemplate(args.format)
        for song in songs:
            print template.format(song)


class OperationCommand(Command):

    def __init__(self, operation):
        self.operation = operation

    def progress_bar(self, songs):
        try:
            from clint.textui import progress
        except ImportError:
            return songs
        else:
            return progress.bar(songs)

    def get_songs_and_progress_bar(self, files):
        return self.progress_bar(self.get_songs(files))

    def apply_operation_and_save(self, operation, files):
        songs = self.get_songs_and_progress_bar(files)
        for song in songs:
            operation.apply(song)
            song.save()

    def run(self, args):
        self.apply_operation_and_save(self.operation, args.files)


class CommandFactory:

    COMMANDS = ['get', 'set', 'rename', 'extract', 'humanize']

    def from_args(self, args):
        name = args.command

        if name == 'get':
            return GetCommand()
        elif name == 'set':
            return OperationCommand(SetOperation(args.__dict__))
        elif name == 'rename':
            template = FreiTemplate(args.format)
            return OperationCommand(RenameOperation(template))
        elif name == 'extract':
            template = FreiTemplate(args.format)
            return OperationCommand(ExtractOperation(template))
        elif name == 'humanize':
            return OperationCommand(HumanizeOperation())



from distutils.core import setup

setup(
    name='freitag',
    version='0.1.1',
    author='G. Capizzi',
    author_email='g.capizzi@gmail.com',
    packages=['freitag'],
    scripts=['bin/freitag'],
    url='https://github.com/gcapizzi/freitag',
    license='COPYING.txt',
    description='A command-line tool for MP3 tagging and renaming.',
    install_requires=[
        "argparse >= 1.2.1",
        "mutagen >= 1.2.0",
    ],
)



#!/usr/bin/env python
# encoding: utf-8
#
# FreiTAG - A simple command line tool to tag and rename mp3s.
# Copyright (c) 2010-2011 Giuseppe Capizzi
# mailto: g.capizzi@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from string import Template
from os import sep
from os.path import exists
from shutil import move
from re import sub, search

from mutagen.mp3 import EasyMP3


DEFAULT_FORMAT = "%tracknumber - %artist - %title.mp3"
TAGS = {
    'album':       {'abbr': 'b', 'help': 'The album name'},
    'artist':      {'abbr': 'a', 'help': 'The artist name'},
    'title':       {'abbr': 't', 'help': 'The track title'},
    'discnumber':  {'abbr': 'd', 'help': 'The disc number'},
    'tracknumber': {'abbr': 'n', 'help': 'The track number'},
    'date':        {'abbr': 'y', 'help': 'The track date (year)'}
}


class FreiTemplate(Template):

    """A custom Template subclass used for tag extraction."""

    delimiter = '%'
    idpattern = '[a-z]+'

    def format(self, values):
        return self.safe_substitute(values).strip()

    def extract(self, string):
        regex = self._template_to_regex(self.template)
        return search(regex, string).groupdict()

    def _tag_to_regex(self, tag):
        tag_regex = '[^%s]*' % sep

        # non-greedy regex for tracknumber tag
        # TODO this is a hack!
        if tag == 'tracknumber':
            tag_regex += '?'

        return '(?P<{0}>{1})'.format(tag, tag_regex)

    def _template_to_regex(self, template):
        # the regex pattern that matches tags in the template string
        tag_pattern = '{0}({1})'.format(self.delimiter, self.idpattern)
        tag_to_regex = lambda m: self._tag_to_regex(m.group(1))
        return sub(tag_pattern, tag_to_regex, template)


class FileSystem:

    def rename(self, old_name, new_name):
        move(old_name, new_name)


class FreiSong:

    """The main FreiTag class, representing a song."""

    def __init__(self, mp3, filesystem=FileSystem()):
        self.mp3 = mp3
        self.prev_filename = mp3.filename
        self.filename = mp3.filename
        self.filesystem = filesystem

    def __getitem__(self, key):
        value = ''

        if key in self.mp3:
            value = self._first(self.mp3[key])

            # remove the slash and everything after it in track number
            # and zero-pad it
            if key == 'tracknumber':
                value = self._fix_tracknumber(value)

        return value

    def _first(self, single_or_list):
        if isinstance(single_or_list, list):
            return single_or_list[0]
        return single_or_list

    def _fix_tracknumber(self, tracknumber):
        return tracknumber.split('/')[0].rjust(2, '0')

    def __contains__(self, tag):
        return tag in self.mp3

    def __setitem__(self, key, value):
        if key in self.mp3:
            self.mp3[key] = value

    def update(self, tags):
        """Update song with tags."""
        tags = self._filter_tags(tags)
        tags = self._unicode_tags(tags)
        self.mp3.update(tags)

    def _filter_tags(self, tags):
        return dict((name, value) for name, value in tags.items()
                    if name in TAGS and value is not None)

    def _unicode_tags(self, tags):
        return dict((name, unicode(value)) for (name, value) in tags.items())

    def save(self):
        """Save the song."""
        self.mp3.save()
        self._rename()

    def _rename(self):
        if self.filename != self.prev_filename and not exists(self.filename):
            self.filesystem.rename(self.prev_filename, self.filename)
            self.prev_filename = self.filename

        self.mp3.filename = self.filename
